/**
 *  Changes log:
 *
 *  2001/04/08  removed check for ' from CAreas::IsValidName(LPCTSTR szName) method
 *
 */
#include "pch.h"

#include "Common.hpp"
#include "Misc.hpp"
#include "Temp.hpp"
#include "CfgParse.hpp"
#include "Areas.h"

CAreas::CAreas() {
	m_pSelectedArea = NULL;
	m_itArea        = m_listAreas.end();
	m_nMaxAreaNameLen = m_nMaxAreaPathLen = 0;
}

CAreas::~CAreas() {
}

void CAreas::Add(SArea &newArea) {
	int	len;

	m_listAreas.push_back( newArea );
	len = lstrlen(newArea.m_szName);
	if (len > m_nMaxAreaNameLen)
		m_nMaxAreaNameLen = len;
	len = lstrlen(newArea.m_szPath);
	if (len > m_nMaxAreaPathLen)
		m_nMaxAreaPathLen = lstrlen(newArea.m_szPath);
}

void CAreas::AutoCreate(LPCTSTR szNewAreaName, SLink* pCreatorLink) {
	int	i;
	TCHAR	szShortName[AREA_NAME_SIZE];
	SArea	newArea;

	lstrcpy( newArea.m_szName, szNewAreaName );
	if (lstrlen(szNewAreaName) > m_nMaxAreaNameLen)
		m_nMaxAreaNameLen = lstrlen(szNewAreaName);
	// set description
	newArea.m_szDesc[0] = _T('\0');

	if (g_vars.m_szAutoDescFileName[0] != _T('\0')) {
		CCfgParser	descParser( g_vars.m_szAutoDescFileName );
		if (!descParser.Failed()) {
			while (!descParser.Eof()) {
				descParser.ReadLine();
				if (descParser.Failed()) {
					if (!descParser.Eof())
						break;
					else
						continue;
				}
				if (descParser.Count() < 3)
					continue;
				if (lstrcmpi( descParser[0], _T("Area") ) == 0 && 
					lstrcmpi( descParser[1], szNewAreaName ) == 0)
				{
					lstrcpyn( newArea.m_szDesc, descParser[2], AREA_DESC_SIZE );
					for (i = 0; newArea.m_szDesc[i] != _T('\0'); i++) {
						if (newArea.m_szDesc[i] == _T('_'))
							newArea.m_szDesc[i] = _T(' ');
					}
					break;
				}
			}
		}
	}

	newArea.m_nRouteType       = ROUTE_NONE;
	newArea.m_nPriority        = 0;
	newArea.m_ulGroup          = pCreatorLink->m_ulDefaultGroup;
	newArea.m_nLine            = -1;
	newArea.m_fPassThrough     = false;
	newArea.m_fVetoManualPurge = false;
	newArea.m_nTraffic         = 0;
	newArea.m_fChanged         = true;

	SLinked	linked;
	linked.m_pLink      = pCreatorLink;
	linked.m_nRouteType =  ROUTE_SEND | pCreatorLink->m_nRouteType;
	newArea.m_listLinked.push_back( linked );

	// form fecho path
	TCHAR	szPath[MAX_PATH];
	if (pCreatorLink->m_szAutoCreatedPathName[0] != _T('\0'))
		lstrcpy( szPath, pCreatorLink->m_szAutoCreatedPathName );
	else
		lstrcpy( szPath, g_vars.m_szAutoCreatedPath );

	if (g_vars.m_fLongPaths)
		_stprintf( newArea.m_szPath, _T("%s%s\\"), szPath, szNewAreaName );
	else {
		i = lstrlen(szNewAreaName);
		if (i > 8)
			i = 8;
		lstrcpyn( szShortName, szNewAreaName, i+1 );
		for (i = 0; szShortName[i] != _T('\0'); i++) {
			if (szShortName[i] == _T('.'))
				szShortName[i] = _T('_');
		}
		_stprintf( newArea.m_szPath, _T("%s%s\\"), szPath, szShortName );
	}
	if (_taccess( newArea.m_szPath, 0 ) == 0) {
		i = 0;
		do {
			if (g_vars.m_fLongPaths)
				_stprintf( newArea.m_szPath, _T("%s%s.%03ld\\"), szPath, szNewAreaName, i );
			else
				_stprintf( newArea.m_szPath, _T("%s%s.%03lX\\"), szPath, szShortName, i );
			i++;
		} while (_taccess( newArea.m_szPath, 0 ) == 0);
	}
	MakeDirStructure( newArea.m_szPath );
	if (lstrlen(newArea.m_szPath) > m_nMaxAreaPathLen)
		m_nMaxAreaPathLen = lstrlen(newArea.m_szPath);
	m_listAreas.push_back( newArea );
	m_pSelectedArea = &m_listAreas.back();
}

bool CAreas::SelectFirst() {
	m_itArea = m_listAreas.begin();
	if (m_itArea == m_listAreas.end()) {
		m_pSelectedArea = NULL;
		return (false);
	}
	m_pSelectedArea = &(*m_itArea);
	return (true);
}

bool CAreas::SelectNext() {
	m_itArea++;
	if (m_itArea == m_listAreas.end()) {
		m_pSelectedArea = NULL;
		return (false);
	}
	m_pSelectedArea = &(*m_itArea);
	return (true);
}

bool CAreas::Select(LPCTSTR szAreaName) {
	for (m_itArea = m_listAreas.begin(); m_itArea != m_listAreas.end(); m_itArea++) {
		if (lstrcmpi( (*m_itArea).m_szName, szAreaName ) == 0)
			break;
	}
	m_pSelectedArea = (m_itArea == m_listAreas.end() ? NULL : (&(*m_itArea)));
	return (m_pSelectedArea != NULL);
}

bool CAreas::CanAccess(SLink* pLink) {
	if (m_pSelectedArea == NULL)
		return (false);
	if (pLink->m_nPriority < m_pSelectedArea->m_nPriority)
		return (false);
	if (m_pSelectedArea->m_ulGroup != 0 && (m_pSelectedArea->m_ulGroup & pLink->m_ulGroups) == 0)
		return (false);
	return (true);
}

SLinked* CAreas::Linked(SLink* pLink) {
	if (m_pSelectedArea == NULL)
		return (NULL);

	vector<SLinked>::iterator	itLinked;

	for (itLinked = m_pSelectedArea->m_listLinked.begin(); 
		itLinked != m_pSelectedArea->m_listLinked.end(); itLinked++)
	{
		if (AddrCmp( pLink->m_addr, (*itLinked).m_pLink->m_addr ) == 0)
			break;
	}
	return (itLinked == m_pSelectedArea->m_listLinked.end() ? NULL : (&(*itLinked)) );
}

bool CAreas::Link(SLink* pLink) {
	if (m_pSelectedArea == NULL)
		return (false);
	if (pLink->m_nPriority < m_pSelectedArea->m_nPriority)
		return (false);
	if (m_pSelectedArea->m_ulGroup != 0 && (m_pSelectedArea->m_ulGroup & pLink->m_ulGroups) == 0)
		return (false);
	SLinked	linked;
	linked.m_pLink = pLink;
	if (m_pSelectedArea->m_nRouteType == ROUTE_NONE)
		linked.m_nRouteType = pLink->m_nRouteType;
	else
		linked.m_nRouteType = m_pSelectedArea->m_nRouteType;
	m_pSelectedArea->m_listLinked.push_back( linked );
	m_pSelectedArea->m_fChanged = true;
	return (true);
}

bool CAreas::Unlink(SLink* pLink) {
	if (m_pSelectedArea == NULL)
		return (false);
	vector<SLinked>::iterator	itLinked;
	for (itLinked = m_pSelectedArea->m_listLinked.begin(); itLinked != m_pSelectedArea->m_listLinked.end(); itLinked++) {
		if (AddrCmp( (*itLinked).m_pLink->m_addr, pLink->m_addr ) == 0)
			break;
	}
	if (itLinked == m_pSelectedArea->m_listLinked.end())
		return (false);
	m_pSelectedArea->m_listLinked.erase(itLinked);
	m_pSelectedArea->m_fChanged = true;
	return (true);
}

bool CAreas::LinksChanged() {
	VAreas::iterator	i;

	for (i = m_listAreas.begin(); i != m_listAreas.end(); i++) {
		if ((*i).m_fChanged)
			return (true);
	}
	return (false);
}

bool CAreas::SaveToFile(LPCTSTR szFileName) {
	TCHAR		szLine[STR_BUF_SIZE];
	TCHAR		szSubLine[STR_BUF_SIZE];
	int		iLineLen, iLen;
	int		iSubLineLen;
	TCHAR		cRoute;
	HANDLE		hFile;
	bool		arrfGroupWritten[26];

	// turn off group written flags

	{
		for (int i = 0; i < 26; i++) {
			arrfGroupWritten[i] = false;
		}
	}

	// open file for writing

	hFile = CreateFile( szFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 
		FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
		return (false);

	VAreas::iterator	itArea;

	ULONG	ulGroup = 0;
	for (itArea = m_listAreas.begin(); itArea != m_listAreas.end(); itArea++) {
		// write CRLF for every new area
		if (itArea != m_listAreas.begin())
			WriteCharString( hFile, _T("\r\n") );

		// write group
		if ((*itArea).m_ulGroup != 0 && (*itArea).m_ulGroup != ulGroup) {
			ulGroup = (*itArea).m_ulGroup;
			TCHAR	cGroup = _T('A');
			ULONG	ulValue = ulGroup;
			while ((ulValue & 1) == 0) {
				ulValue >>= 1;
				cGroup++;
			}
			tstring&	sGroupName = m_arrsGroupNames[cGroup - _T('A')];
			if (sGroupName.length() > 0) {
				_stprintf( szLine, _T("Group %c \"%s\"\r\n\r\n"), cGroup, sGroupName.c_str() );
				WriteCharString( hFile, szLine );
				arrfGroupWritten[cGroup - _T('A')] = true;
			}
		}

		// determine a route type
		switch ((*itArea).m_nRouteType) {
		case ROUTE_NONE:
			cRoute = _T(' ');
			break;
		case ROUTE_RECEIVE:
			cRoute = _T('R');
			break;
		case ROUTE_SEND:
			cRoute = _T('S');
			break;
		case ROUTE_BIDIRECT:
			cRoute = _T('B');
			break;
		}

		// prepare the 1-st line

		// name
		ConcatStrings( szLine, _T("Area "), (*itArea).m_szName );
		iLineLen = lstrlen( szLine );
		iLen = lstrlen( (*itArea).m_szName );
		while (iLen < m_nMaxAreaNameLen) {
			szLine[iLineLen++] = _T(' ');
			iLen++;
		}
		// path
		ConcatStrings( szLine + iLineLen, _T(" "), (*itArea).m_szPath );
		iLineLen = lstrlen( szLine );
		iLen = lstrlen( (*itArea).m_szPath );
		while (iLen < m_nMaxAreaPathLen) {
			szLine[iLineLen++] = _T(' ');
			iLen++;
		}
		// route
		_stprintf( szLine + iLineLen, _T(" %c"), cRoute );
		iLineLen += 2;
		// passthrough
		if ((*itArea).m_fPassThrough) {
			lstrcpy( szLine + iLineLen, _T(" P") );
			iLineLen += 2;
		}
		// veto
		if ((*itArea).m_fVetoManualPurge) {
			lstrcpy( szLine + iLineLen, _T(" V") );
			iLineLen += 2;
		}
		// priority/group
		if ((*itArea).m_ulGroup != 0) {
			_stprintf( szLine + iLineLen, _T(" %ld/%c"), (*itArea).m_nPriority,
				(TCHAR)GetOnBit((*itArea).m_ulGroup) + _T('A') );
		}
		else if ((*itArea).m_nPriority != 0)
			_stprintf( szLine + iLineLen, _T(" %ld"), (*itArea).m_nPriority );
		iLineLen = lstrlen( szLine );
		if ((*itArea).m_szDesc[0] != _T('\0'))
			_stprintf( szLine + iLineLen, _T(" \"%s\""), (*itArea).m_szDesc );
		lstrcat( szLine, _T("\r\n") );
		WriteCharString( hFile, szLine );

		// Linked

		lstrcpy( szLine, _T("Linked") );

		vector<SLinked>::iterator	itLinked;

		for (itLinked = (*itArea).m_listLinked.begin(); itLinked != (*itArea).m_listLinked.end(); /* none */) {
			iLineLen = lstrlen( szLine );
			if ((*itLinked).m_nRouteType != (*itLinked).m_pLink->m_nRouteType) {
				switch ((*itLinked).m_nRouteType) {
				case ROUTE_RECEIVE:
					cRoute = _T('R');
					break;
				case ROUTE_SEND:
					cRoute = _T('S');
					break;
				case ROUTE_BIDIRECT:
					cRoute = _T('B');
					break;
				}
				_stprintf( szSubLine, _T(" %c"), cRoute );
			}
			else
				lstrcpy( szSubLine, _T(" ") );
			iSubLineLen = lstrlen( szSubLine );
			if (lstrcmpi( g_vars.m_addrMain.Domain, (*itLinked).m_pLink->m_addr.Domain ) != 0) {
				(*itLinked).m_pLink->m_addr.ToString( szSubLine + iSubLineLen );
			}
			else {
				if ((*itLinked).m_pLink->m_addr.Zone == g_vars.m_addrMain.Zone) {
					if ((*itLinked).m_pLink->m_addr.Net == g_vars.m_addrMain.Net) {
						_stprintf( szSubLine + iSubLineLen, _T("%ld"), 
							(*itLinked).m_pLink->m_addr.Node );
					}
					else {
						_stprintf( szSubLine + iSubLineLen, _T("%ld/%ld"),
							(*itLinked).m_pLink->m_addr.Net,
							(*itLinked).m_pLink->m_addr.Node );
					}
				}
				else {
					_stprintf( szSubLine + iSubLineLen, _T("%ld:%ld/%ld"),
						(*itLinked).m_pLink->m_addr.Zone,
						(*itLinked).m_pLink->m_addr.Net,
						(*itLinked).m_pLink->m_addr.Node );
				}
				if ((*itLinked).m_pLink->m_addr.Point != 0) {
					iSubLineLen = lstrlen( szSubLine );
					_stprintf( szSubLine + iSubLineLen, _T(".%ld"),
						(*itLinked).m_pLink->m_addr.Point );
				}
			}

			if (lstrlen(szLine) + lstrlen(szSubLine) > 80) {
				lstrcpy( szLine + iLineLen, _T("\r\n") );
				WriteCharString( hFile, szLine );
				szLine[6] = _T('\0'); // now szLine == _T("Linked")
			}
			else {
				lstrcat( szLine, szSubLine );
				itLinked++;
			}
		}
		if (lstrlen(szLine) > 6) {
			lstrcat( szLine, _T("\r\n") );
			WriteCharString( hFile, szLine );
		}
	}

	WriteCharString( hFile, _T("\r\n") );

	// write unwritten groups
	{
		for (int i = 0; i < 26; i++) {
			if (!arrfGroupWritten[i] && m_arrsGroupNames[i].length() > 0) {
				_stprintf( szLine, _T("Group %c \"%s\"\r\n"), _T('A') + i, m_arrsGroupNames[i].c_str() );
				WriteCharString( hFile, szLine );
			}
		}
	}

	CloseHandle( hFile );
	return (true);
}


void CAreas::GetNameDesc(LPTSTR szNameDesc) {
	int	iLen;
	TCHAR	szDesc[1024];
    
        if (m_pSelectedArea == NULL)
                *szNameDesc = _T('\0');
        else {
                lstrcpy( szNameDesc, _T("  ") );
                lstrcat( szNameDesc, m_pSelectedArea->m_szName );
                iLen = lstrlen( szNameDesc );
                szNameDesc[iLen++] = _T(' ');
                while (iLen < 39)
                	szNameDesc[iLen++] = _T('.');
                szNameDesc[iLen++] = _T(' ');
                if (m_pSelectedArea->m_szDesc[0] != _T('\0')) {
                	AlignDescription( m_pSelectedArea->m_szDesc, szDesc, 38, 40 );
                	lstrcpy( szNameDesc + iLen, szDesc );
                }
                else
                	lstrcpy( szNameDesc + iLen, _T("no description") );
        }
}

void CAreas::AlignDescription(LPCTSTR szDesc, LPTSTR szAligned, int iLen, int iPrefixLen) {
	int	i, j, pos;
	int	wordStart, wordLen;
	int	index;

	i   = 0;
	j   = 0;
	pos = 0;
	wordStart = -1;
	while (szDesc[i] != _T('\0')) {
		if (pos == iLen) {
			szAligned[j] = _T('\r');
			j++;
			for (index = 0; index < iPrefixLen; index++)
				szAligned[j + index] = _T(' ');
			j += iPrefixLen;
			pos = 0;
		}
		if (szDesc[i] == _T(' ')) {
			if (wordStart >= 0) {
				if (wordLen > iLen) {
					::CopyMemory( szAligned + j, szDesc + wordStart, (iLen - pos)*sizeof(TCHAR) );
					j += iLen - pos;
					szAligned[j] = _T('\r');
					j++;
					wordStart += iLen - pos;
					wordLen -= iLen - pos;
					while (wordLen > iLen) {
						for (index = 0; index < iPrefixLen; index++)
							szAligned[j + index] = _T(' ');
						j += iPrefixLen;
						::CopyMemory( szAligned + j, szDesc + wordStart, iLen*sizeof(TCHAR) );
						j += iLen;
						szAligned[j] = _T('\r');
						j++;
						wordLen -= iLen;
						wordStart += iLen;
					}
					if (wordLen > 0) {
						for (index = 0; index < iPrefixLen; index++)
							szAligned[j + index] = _T(' ');
       						j += iPrefixLen;
						::CopyMemory( szAligned + j, szDesc + wordStart, wordLen*sizeof(TCHAR) );
						j += wordLen;
					}
					pos = wordLen;
				}
				else {
					if (pos + wordLen < iLen) {
						::CopyMemory( szAligned + j, szDesc + wordStart, wordLen*sizeof(TCHAR) );
        					j += wordLen;
        					pos += wordLen;
					}
					else {
						for (index = 0; index < iLen - pos; index++)
							szAligned[j + index] = _T(' ');
						j += (iLen - pos);
						szAligned[j] = _T('\r');
						j++;
						for (index = 0; index < iPrefixLen; index++)
							szAligned[j + index] = _T(' ');
						j += iPrefixLen;
						::CopyMemory( szAligned + j, szDesc + wordStart, wordLen*sizeof(TCHAR) );
						j += wordLen;
						pos = wordLen;
					}
				}
				wordStart = -1;
			}
			if (pos > 0 && pos < iLen) {
				szAligned[j] = _T(' ');
				j++;
				pos++;
			}
		}
		else {
			if (wordStart < 0) {
				wordStart = i;
				wordLen   = 1;
			}
			else
				wordLen++;
		}
		i++;
	}
 	if (wordStart >= 0) {
 		if (wordLen > iLen) {
			::CopyMemory( szAligned + j, szDesc + wordStart, (iLen - pos)*sizeof(TCHAR) );
			j += iLen - pos;
			szAligned[j] = _T('\r');
			j++;
			wordStart += iLen - pos;
			wordLen -= iLen - pos;
 			while (wordLen > iLen) {
				for (index = 0; index < iPrefixLen; index++)
					szAligned[j+index] = _T(' ');
 				j += iPrefixLen;
				::CopyMemory( szAligned + j, szDesc + wordStart, iLen*sizeof(TCHAR) );
 				j += iLen;
 				szAligned[j] = _T('\r');
 				j++;
 				wordLen -= iLen;
 				wordStart += iLen;
 			}
 			if (wordLen > 0) {
				for (index = 0; index < iPrefixLen; index++)
					szAligned[j + index] = _T(' ');
				j += iPrefixLen;
				::CopyMemory( szAligned + j, szDesc + wordStart, wordLen*sizeof(TCHAR) );
 				j += wordLen;
 			}
 			pos = wordLen;
 		}
 		else {
 			if (pos + wordLen < iLen) {
				::CopyMemory( szAligned + j, szDesc + wordStart, wordLen*sizeof(TCHAR) );
 				j += wordLen;
 				pos += wordLen;
 			}
 			else {
				for (index = 0; index < (iLen - pos); index++)
 					szAligned[j+index] = _T(' ');
 				j += (iLen - pos);
 				szAligned[j] = _T('\r');
 				j++;
				for (index = 0; index < iPrefixLen; index++)
 					szAligned[j+index] = _T(' ');
 				j += iPrefixLen;
				::CopyMemory( szAligned + j, szDesc + wordStart, wordLen*sizeof(TCHAR) );
 				j += wordLen;
 				pos = wordLen;
 			}
 		}
 	}
	szAligned[j] = _T('\0');
}

int CAreas::GetRouteType(const SLink* pLink) const {
	if (m_pSelectedArea == NULL)
		return (ROUTE_NONE);

	vector<SLinked>::iterator	itLinked;
	for (itLinked = m_pSelectedArea->m_listLinked.begin(); itLinked != m_pSelectedArea->m_listLinked.end(); itLinked++) {
		if (AddrCmp( (*itLinked).m_pLink->m_addr, pLink->m_addr ) == 0)
			return ((*itLinked).m_nRouteType);
	}
	return (ROUTE_NONE);
}

bool CAreas::UpdateDescFile(LPCTSTR szFileName, LPCTSTR szFileDesc, filesize_t fileSize,
                           vector<tstring> &listLDesc)
{
	ofstream     out;
	ifstream     in;
	ofstream     tempout;
	TCHAR        szDescFileName[MAX_PATH];
	TCHAR        szTempFileName[MAX_PATH];
	TCHAR        szWorkFileName[MAX_PATH];
	TCHAR        szBuf[LINE_SIZE];
	char		szOemBuf[LINE_SIZE];
	char		szOemFileName[MAX_PATH];
	int          Len;
	int          i;
	bool         fSkipLongDesc;
    
	if (m_pSelectedArea == NULL)
		return (false);

	// set file names
	lstrcpy( szDescFileName, m_pSelectedArea->m_szPath );
	lstrcat( szDescFileName, g_vars.m_szFilelistFileName );
	lstrcpy( szTempFileName, m_pSelectedArea->m_szPath );
	lstrcat( szTempFileName, _T("adrenalin.tmp") );
	Len = lstrlen( szFileName );

	// check name for spaces
	for (i = 0; szFileName[i] != _T('\0'); i++) {
		if (szFileName[i] == _T(' '))
			break;
	}
	if (szFileName[i] != _T('\0')) {
		lstrcpy( szWorkFileName, _T("\"") );
		lstrcat( szWorkFileName, szFileName );
		lstrcat( szWorkFileName, _T("\"") );
		Len += 2;
	}
	else
		lstrcpy( szWorkFileName, szFileName );

	// copy desc file into temp file, but skip lines with WorkFileName
	BOOL fSuccessOverwrite = TRUE;

	do {
		// check temp file name
		if (!GetTmpFileName( szTempFileName ))
			break;
		CharToOem( szDescFileName, szOemFileName );
		SetFileApisToOEM();
		in.open( szOemFileName );
		if (!in) {
			SetFileApisToANSI();
			break;
		}
		CharToOem( szTempFileName, szOemFileName );
		tempout.open( szOemFileName );
		if (!tempout) {
			in.close();
			SetFileApisToANSI();
			break;
		}
		SetFileApisToANSI();
		CharToOem( szWorkFileName, szOemFileName );
		fSkipLongDesc = false;
		while (!in.eof()) {
			in.getline( szOemBuf, LINE_SIZE );
			if (!in) {
				if (!in.eof())
					break;
				continue;
			}

			// skip empty lines
			if (*szOemBuf == '\0')
				continue;

			if ((strnicmp( szOemBuf, szOemFileName, Len) == 0) && ((szOemBuf[Len] == ' ') || (szOemBuf[Len] == '\0')))
				fSkipLongDesc = true;
			else {
				if (fSkipLongDesc) {
					if (*szOemBuf != ' ' && *szOemBuf != '>' && *szOemBuf != '+') {
						fSkipLongDesc = false;
						tempout << szOemBuf << endl;
						if (!tempout) {
							fSuccessOverwrite = false;
							break;
						}
					}
				} else {
					tempout << szOemBuf << endl;
					if (!tempout) {
						fSuccessOverwrite = false;
						break;
					}
				}
			}
		}
		in.close();
		tempout.close();
		if (fSuccessOverwrite) {
			_tunlink( szDescFileName );
			_trename( szTempFileName, szDescFileName );
		} else {
			_tunlink( szTempFileName );
			g_vars.m_log << _T("! error updating ") << szDescFileName << _T("\r\n");
		}
		break;
	} while (true);
    
	CharToOem( szDescFileName, szOemFileName );
	out.open( szOemFileName, ios::app );
	if (!out)
	        out.open( szOemFileName );
	if (!out)
		return (0);
	CharToOem( szWorkFileName, szOemFileName );

	CharToOem( g_vars.m_fileListFormat.apply( szFileName, szFileDesc, fileSize).c_str(), szOemBuf );

	/*
	out << szOemFileName;
	if (*szFileDesc != _T('\0')) {
		while (Len < 12) {
			out << " ";
			Len++;
		}
		out << " ";
		CharToOem( szFileDesc, szOemBuf );
		out << szOemBuf;
	}
	*/
	out << szOemBuf;
	out << endl;


	vector<tstring>::iterator	it;
	for (it = listLDesc.begin(); it != listLDesc.end(); it++) {
		ConcatStrings( szBuf, g_vars.m_szLongDescPrefix, (*it).c_str() );
		CharToOem( szBuf, szOemBuf );
		out << szOemBuf << endl;
	}
	out.close();
	return (true);
}

bool CAreas::IsValidName(LPCTSTR szName) {
	while (*szName != _T('\0')) {
		if (*szName == _T('\\') || *szName == _T('/') || *szName == _T(':') ||
			*szName == _T('*') || *szName == _T('?') ||
			*szName == _T('<') || *szName == _T('>') ||
			*szName == _T('"'))
			return (false);
		szName++;
	}
	return (true);
}

void CAreas::ClearDescFile(VTString& vsFileNames) {
	ifstream     in;
	ofstream     tempout;
	TCHAR        szDescFileName[MAX_PATH];
	TCHAR        szTempFileName[MAX_PATH];
	TCHAR        szBuf[LINE_SIZE];
	char		szOemFileName[MAX_PATH];
	char		szOemBuf[MAX_PATH];
	bool         fSkipLongDesc;
    
	if (m_pSelectedArea == NULL)
		return;

	// set file names
	lstrcpy( szDescFileName, m_pSelectedArea->m_szPath );
	lstrcat( szDescFileName, g_vars.m_szFilelistFileName );
	lstrcpy( szTempFileName, m_pSelectedArea->m_szPath );
	lstrcat( szTempFileName, _T("adrenalin.tmp") );

	// enclose names with spaces in quotes

	VTString	vsWorkFileNames;
	VTString::iterator	itName;

	for (itName = vsFileNames.begin(); itName != vsFileNames.end(); itName++) {
		if (HasSpaces((*itName).c_str())) {
			vsWorkFileNames.push_back( tstring(_T("\"")) + (*itName) + _T("\"") );
		}
		else {
			vsWorkFileNames.push_back( *itName );
		}
	}

	// copy desc file into temp file, but skip lines with WorkFileName
	do {
		// check temp file name
		if (!GetTmpFileName( szTempFileName ))
			break;
		SetFileApisToOEM();
		CharToOem( szDescFileName, szOemFileName );
		in.open( szOemFileName );
		if (!in) {
			SetFileApisToANSI();
			break;
		}
		CharToOem( szTempFileName, szOemFileName );
		tempout.open( szOemFileName );
		if (!tempout) {
			in.close();
			SetFileApisToANSI();
			break;
		}
		// prepare buffer for new line
		fSkipLongDesc = false;
		while (!in.eof()) {
			in.getline( szOemBuf, LINE_SIZE );
			if (!in)
				continue;
			OemToChar( szOemBuf, szBuf );
			// find name
			for (itName = vsWorkFileNames.begin(); itName != vsWorkFileNames.end(); itName++) {
				if (_tcsnicmp( szBuf, (*itName).c_str(), (*itName).length() ) == 0)
					break;
			}
			if (itName != vsWorkFileNames.end())
				fSkipLongDesc = true;
			else {
				if (fSkipLongDesc) {
					if (*szOemBuf != ' ' && *szOemBuf != '>' && 
						*szOemBuf != '+') {
						fSkipLongDesc = false;
						tempout << szOemBuf << endl;
					}
				} else
					tempout << szOemBuf << endl;
			}
		}
		in.close();
		tempout.close();
		_tunlink( szDescFileName );
		_trename( szTempFileName, szDescFileName );
		break;
	} while (true);
}


void CAreas::ClearDescFile(LPCTSTR szFileName) {
	ifstream     in;
	ofstream     tempout;
	TCHAR        szDescFileName[MAX_PATH];
	TCHAR        szTempFileName[MAX_PATH];
	TCHAR        szWorkFileName[MAX_PATH];
	TCHAR        szBuf[LINE_SIZE];
	char		szOemFileName[MAX_PATH];
	char		szOemBuf[MAX_PATH];
	int          Len;
	int          i;
	bool         fSkipLongDesc;
    
	if (m_pSelectedArea == NULL)
		return;

	// set file names
	lstrcpy( szDescFileName, m_pSelectedArea->m_szPath );
	lstrcat( szDescFileName, g_vars.m_szFilelistFileName );
	lstrcpy( szTempFileName, m_pSelectedArea->m_szPath );
	lstrcat( szTempFileName, _T("adrenalin.tmp") );
	Len = lstrlen( szFileName );

	// check name for spaces
	for (i = 0; szFileName[i] != _T('\0'); i++) {
		if (szFileName[i] == _T(' '))
			break;
	}
	if (szFileName[i] != _T('\0')) {
		lstrcpy( szWorkFileName, _T("\"") );
		lstrcat( szWorkFileName, szFileName );
		lstrcat( szWorkFileName, _T("\"") );
		Len += 2;
	}
	else
		lstrcpy( szWorkFileName, szFileName );
	// copy desc file into temp file, but skip lines with WorkFileName
	do {
		// check temp file name
		if (!GetTmpFileName( szTempFileName ))
			break;
		SetFileApisToOEM();
		CharToOem( szDescFileName, szOemFileName );
		in.open( szOemFileName );
		if (!in) {
			SetFileApisToANSI();
			break;
		}
		CharToOem( szTempFileName, szOemFileName );
		tempout.open( szOemFileName );
		if (!tempout) {
			in.close();
			SetFileApisToANSI();
			break;
		}
		CharToOem( szWorkFileName, szOemFileName );
		fSkipLongDesc = false;
		while (!in.eof()) {
			in.getline( szOemBuf, LINE_SIZE );
			if (!in)
				continue;
			if (strnicmp( szBuf, szOemFileName, Len) == 0)
				fSkipLongDesc = true;
			else {
				if (fSkipLongDesc) {
					if (*szOemBuf != ' ' && *szOemBuf != '>' && 
						*szOemBuf != '+') {
						fSkipLongDesc = false;
						tempout << szOemBuf << endl;
					}
				} else
					tempout << szOemBuf << endl;
			}
		}
		in.close();
		tempout.close();
		_tunlink( szDescFileName );
		_trename( szTempFileName, szDescFileName );
		break;
	} while (true);
}

void CAreas::Sort() {
	TCHAR	szSelAreaName[AREA_NAME_SIZE];

	// memorize selected area
	if (m_pSelectedArea != NULL)
		lstrcpy( szSelAreaName, m_pSelectedArea->m_szName );
	else
		*szSelAreaName = _T('\0');

	sort( m_listAreas.begin(), m_listAreas.end() );

	// restore selected area
	if (*szSelAreaName != _T('\0'))
		Select( szSelAreaName );
}

SArea& SArea::operator =(const SArea &area) {
	m_nRouteType       = area.m_nRouteType;
	m_nPriority        = area.m_nPriority;
	m_ulGroup          = area.m_ulGroup;
	m_fPassThrough     = area.m_fPassThrough;
	m_fVetoManualPurge = area.m_fVetoManualPurge;
	m_listLinked       = area.m_listLinked;
	m_nLine            = area.m_nLine;
	m_fChanged         = area.m_fChanged;
	m_nTraffic         = area.m_nTraffic;
	m_vsFileFlags      = area.m_vsFileFlags;
	m_vsFileMasks      = area.m_vsFileMasks;
	m_vbFileFlagsUsed  = area.m_vbFileFlagsUsed;
	m_vsNewFiles       = area.m_vsNewFiles;
	lstrcpy( m_szName, area.m_szName );
	lstrcpy( m_szPath, area.m_szPath );
	lstrcpy( m_szDesc, area.m_szDesc );
	return (*this);
}

bool operator < (const SArea a1, const SArea a2) {
	// compare areas groups
	if (a1.m_ulGroup == a2.m_ulGroup) {
		// make result based on names comparision
		return (lstrcmpi( a1.m_szName, a2.m_szName ) < 0);
	}
	// make result based on groups comparision
	return (a1.m_ulGroup < a2.m_ulGroup);                 
}

tstring& CAreas::GetGroupName(ULONG ulGroup) {
	// ulGroup should not be zero
	assert( ulGroup != 0 );
	// get index from group bit
	int	iGroup = 0;
	while ((ulGroup & 1) == 0) {
		iGroup++;
		ulGroup >>= 1;
	}
	assert( iGroup < 26 );
	return (m_arrsGroupNames[iGroup]);
}
