#ifndef __ADRENALIN__AREAS__HEADER__
#define __ADRENALIN__AREAS__HEADER__

#include "Links.hpp"
#include "Common.hpp"
#include "Flag.h"

const
    AREA_NAME_SIZE = 32,
    AREA_DESC_SIZE = 256;


struct SLinked {
	SLink*	m_pLink;
	int	m_nRouteType;
};

typedef	vector<SLinked>	VLinked;

struct SArea {
	SArea() :
		m_nRouteType(0),
		m_nPriority(0),
		m_ulGroup(0),
		m_nLine(0),
		m_nTraffic(0)
		{
			m_szName[0] = _T('\0');
			m_szPath[0] = _T('\0');
			m_szDesc[0] = _T('\0');
		}

	SArea(const SArea &area) :
		m_nRouteType(area.m_nRouteType),
		m_nPriority(area.m_nPriority),
		m_ulGroup(area.m_ulGroup),
		m_fPassThrough(area.m_fPassThrough),
		m_fVetoManualPurge(area.m_fVetoManualPurge),
		m_listLinked(area.m_listLinked),
		m_nLine(area.m_nLine),
		m_fChanged(area.m_fChanged),
		m_nTraffic(area.m_nTraffic),
		m_vsFileFlags(area.m_vsFileFlags),
		m_vsFileMasks(area.m_vsFileMasks),
		m_vbFileFlagsUsed(area.m_vbFileFlagsUsed),
		m_vsNewFiles(area.m_vsNewFiles)
		{
			lstrcpy( m_szName, area.m_szName );
			lstrcpy( m_szPath, area.m_szPath );
			lstrcpy( m_szDesc, area.m_szDesc );
		}
	SArea& operator =(const SArea &area);
	TCHAR		m_szName[AREA_NAME_SIZE];
	TCHAR		m_szPath[MAX_PATH];
	TCHAR		m_szDesc[AREA_DESC_SIZE];
	int		m_nRouteType;
	int		m_nPriority;
	unsigned long	m_ulGroup;
	CFlag		m_fPassThrough;
	CFlag		m_fVetoManualPurge;
	VLinked		m_listLinked;
	int		m_nLine;
	CFlag		m_fChanged;
	int		m_nTraffic;

	VTString	m_vsFileFlags;
	VTString	m_vsFileMasks;
	vector<bool>	m_vbFileFlagsUsed;
	VTString	m_vsNewFiles;
};

bool operator < (const SArea a1, const SArea a2);

typedef vector<SArea>	VAreas;

class CAreas {
public:
	CAreas();
	~CAreas();
	void     Add(SArea &newArea);
	void     AutoCreate(LPCTSTR szNewAreaName, SLink* pCreatorLink);

	bool		SelectFirst();
	bool		SelectNext();
	bool		Select(LPCTSTR szAreaName);
	inline SArea*	GetSelectedArea() const { return (m_pSelectedArea); }

	bool     CanAccess(SLink* pLink);
	SLinked* Linked(SLink* pLink);
	bool     Link(SLink* pLink);
	bool     Unlink(SLink* pLink);
	bool     LinksChanged();
	bool     SaveToFile(LPCTSTR szFileName);
	void     GetNameDesc(LPTSTR szNameDesc);
	int      GetRouteType(const SLink* pLink) const;
	bool     UpdateDescFile(LPCTSTR szFileName, LPCTSTR szFileDesc, filesize_t fileSize,
			    vector<tstring> &listLDesc);
	void     ClearDescFile(LPCTSTR szFileName);
	void	ClearDescFile(VTString& vsFileNames);
	bool     IsValidName(LPCTSTR szName);
	void     Sort();
	void     GetAreas();
	tstring& GetGroupName(ULONG ulGroup);
private:
	VAreas			m_listAreas;
	VAreas::iterator	m_itArea;
	SArea*			m_pSelectedArea;
	int			m_nMaxAreaNameLen, m_nMaxAreaPathLen;
	tstring			m_arrsGroupNames[26];
	void			AlignDescription(LPCTSTR szDesc, LPTSTR szAligned, int iLen, int iPrefixLen);
};

#endif
