#include "pch.h"

#include "Mem.hpp"
#include "Common.hpp"
#include "CfgParse.hpp"
#include "Misc.hpp"
#include "Links.hpp"
#include "Areas.h"
#include "Msg.hpp"
#include "Tic.hpp"
#include "crc32.hpp"
#include "Temp.hpp"
#include "File.h"

#include "CfgLexems.hpp"
#include "WELexems.h"

#include "MsgAttrs.h"

#include "Adrenalin.h"

/**
 * Implementation of void CAdrenalin::PrintDebugInfo()
 */

void CAdrenalin::PrintDebugInfo() {
	int		i;
	unsigned long	ulBit;
	TCHAR		szChar[2];

	TCHAR	szBuf[1024];

	g_vars.m_log << CLog::Push(new CLog::FileOut(false))
		     << CLog::Push(new CLog::StdErr(false))
		     << CLog::Push(new CLog::StdOut(true));

	g_vars.m_log << _T("Debug information\n");

	g_vars.m_addrMain.ToString( szBuf );
	g_vars.m_log << _T("System: ") << szBuf << _T("\n");
	for (i = 0; i < g_vars.m_vAKAAddr.size(); i++) {
		g_vars.m_vAKAAddr[i].ToString( szBuf );
		g_vars.m_log << _T("   AKA: ") << szBuf << _T("\n");
	}
	g_vars.m_log << _T(" SysOp: ") << g_vars.m_szSysOp << _T("\n");

	g_vars.m_log << _T("Directories:\n");
	g_vars.m_log << _T("       Inbound: ") << g_vars.m_szInboundPath      << TEXT_EOL
		     << _T("      Outbound: ") << g_vars.m_szOutboundPath     << TEXT_EOL
		     << _T("  BinkOutbound: ") << g_vars.m_szBinkOutboundPath << TEXT_EOL
		     << _T("       Netmail: ") << g_vars.m_szNetmailPath      << TEXT_EOL
		     << _T("       BadTics: ") << g_vars.m_szBadTicsPath      << TEXT_EOL
		     << _T("       NewTics: ") << g_vars.m_szNewTicsPath      << TEXT_EOL
		     << _T(" ProcessedTics: ");
	if (g_vars.m_szProcessedTicsPath[0] == TEXT('\0')) {
		g_vars.m_log << TEXT("undefined (TICs will be removed after processing)\n");
	}
	else {
		g_vars.m_log << g_vars.m_szProcessedTicsPath << TEXT_EOL;
	}
    
	g_vars.m_log << TEXT("      Filelist: ") << g_vars.m_szFilelistFileName << TEXT_EOL;
	g_vars.m_log << TEXT("FilelistFormat: ") << g_vars.m_fileListFormat.getFormat() << TEXT_EOL;

    // requests log file
    g_vars.m_log << TEXT("Requests log: ")
                 << (IsEmptyStr(requestsLogFileName) ? TEXT("undefined")
                                                     : requestsLogFileName)
                 << TEXT_EOL;

	g_vars.m_log << _T("Flags:\n");
	g_vars.m_log << _T("    after fix: ")
		     << ((*g_vars.m_szFixFlagName != '\0') ? g_vars.m_szFixFlagName : _T("none"))
		     << _T("\n")
		     << _T("   after toss: ")
		     << ((*g_vars.m_szTossFlagName != '\0') ? g_vars.m_szTossFlagName : _T("none"))
		     << _T("\n")
		     << _T("  after hatch: ")
		     << ((*g_vars.m_szHatchFlagName != '\0') ? g_vars.m_szHatchFlagName : _T("none"))
		     << _T("\n");
    
	g_vars.m_log << _T("Alias(es):\n");
	for (i = 0; i < g_vars.m_vsAliases.size(); i++) {
		g_vars.m_log << _T("  ") << g_vars.m_vsAliases[i] << _T("\n");
	}

	// attributes
	g_vars.m_log << TEXT("Reply message attributes: ") 
		     << MsgAttrsToString( g_vars.m_nReplyMessageAttrs,
					  g_vars.m_nReplyMessageFlags )
		     << TEXT_EOL;

	// warn sysop
	g_vars.m_log << TEXT("Warn sysop by mail about:");
	if (g_vars.m_nWarnSysopByMail == 0) {
		g_vars.m_log << TEXT( " nothing" );
	}
	else {
		if ((g_vars.m_nWarnSysopByMail & WARN_SYSOP_ABOUT_WARNINGS) != 0) {
			g_vars.m_log << TEXT( " warnings" );
		}
		if ((g_vars.m_nWarnSysopByMail & WARN_SYSOP_ABOUT_ERRORS) != 0) {
			g_vars.m_log << TEXT( " errors" );
		}
	}
	g_vars.m_log << TEXT_EOL;

	// links
	g_vars.m_log << _T("Link(s):\n");
	if (m_links.SelectFirst()) {
		do {
			SLink&	link = m_links.GetSelected();
			link.m_addr.ToString( szBuf );
			g_vars.m_log << _T("  --------------\n")
				     << _T("        address: ") << szBuf << _T("\n")
				     << _T("  --------------\n")
				     << _T("        CfgFile: ") << link.m_szCfgFileName << _T("\n")
				     << _T("           line: ") << link.m_nLine << _T("\n")
				     << _T("       password: ") << link.m_szPassword << _T("\n")
				     << _T("          route: ");
			switch (link.m_nRouteType) {
			case ROUTE_SEND:
				g_vars.m_log << _T("to us");
				break;
			case ROUTE_RECEIVE:
				g_vars.m_log << _T("from us");
				break;
			case ROUTE_BIDIRECT:
				g_vars.m_log << _T("to us/from us");
				break;
			}
			g_vars.m_log << _T("\n")
				     << _T("            BSO: ");
			switch (link.m_nBSOType) {
			case BSO_NORMAL:
				g_vars.m_log << _T("normal");
				break;
			case BSO_CRASH:
				g_vars.m_log << _T("crash");
				break;
			case BSO_DIRECT:
				g_vars.m_log << _T("direct");
				break;
			case BSO_HOLD:
				g_vars.m_log << _T("hold");
				break;
			case BSO_IMMEDIATE:
				g_vars.m_log << _T("immediate");
				break;
			}
			g_vars.m_log << _T("\n")
				     << _T("  area creation: ");
			if (!link.m_fAllowAreaCreation)
				g_vars.m_log << _T("not ");
			g_vars.m_log << _T("allowed\n")
				     << _T("       priority: ") << link.m_nPriority << _T("/");

			ulBit  = 1;
			lstrcpy( szChar, _T("A") );
			while (*szChar <= _T('Z')) {
				if (link.m_ulGroups & ulBit)
					g_vars.m_log << szChar;
				ulBit <<= 1;
				(*szChar)++;
			}
			g_vars.m_log << _T("\n");
			g_vars.m_log << _T("  default group: ");
			if (link.m_ulDefaultGroup == 0)
				g_vars.m_log << _T("no\n");
			else {
				ulBit  = 1;
				lstrcpy( szChar, _T("A") );
				while (*szChar <= _T('Z')) {
					if (link.m_ulDefaultGroup & ulBit) {
						g_vars.m_log << szChar << _T("\n");
						break;
					}
					ulBit <<= 1;
					(*szChar)++;
				}
			}
			if (link.m_iOurAddr == MAIN_ADDRESS)
				g_vars.m_addrMain.ToString( szBuf );
			else
				g_vars.m_vAKAAddr[link.m_iOurAddr].ToString( szBuf );
			g_vars.m_log << _T("   our address: ") << szBuf << _T("\n")
				     << _T("    AvailAreas: ") << link.m_szAvailAreasFileName << _T("\n");
		} while (m_links.SelectNext());
		
	}
	g_vars.m_log << _T("Area(s):\n");
	if (m_areas.SelectFirst()) {
		do {
			SArea&	area = *m_areas.GetSelectedArea();
			g_vars.m_log << _T("---\n");
			g_vars.m_log << _T("    name: ") << area.m_szName << _T("\n")
				     << _T("    path: ") << area.m_szPath << _T("\n")
				     << _T("    desc: ") << area.m_szDesc << _T("\n")
				     << _T("   route: ") << area.m_nRouteType << _T("\n")
				     << _T("  access: ") << area.m_nPriority << _T("/");
			if (area.m_ulGroup == 0)
				g_vars.m_log << _T("any\n");
			else {
				szChar[0] = _T('A') + GetOnBit(area.m_ulGroup);
				szChar[1] = _T('\0');
				g_vars.m_log << szChar << _T("\n");
			}
			g_vars.m_log << _T("  passthrough: ");
			if (area.m_fPassThrough)
				g_vars.m_log << _T("yes\n");
			else
				g_vars.m_log << _T("no\n");
			g_vars.m_log << _T("  veto manual purge: ");
			if (area.m_fVetoManualPurge)
				g_vars.m_log << _T("yes\n");
			else
				g_vars.m_log << _T("no\n");
			VLinked::iterator	itLinked = area.m_listLinked.begin();
			if (itLinked == area.m_listLinked.end())
				g_vars.m_log << _T("  none links linked\n");
			else {
				g_vars.m_log << _T("  linked:\n");
				do {
					(*itLinked).m_pLink->m_addr.ToString( szBuf );
					g_vars.m_log << _T("  ") << szBuf << _T(", ")
						     << (*itLinked).m_nRouteType << _T("\n");
					itLinked++;
				} while (itLinked != area.m_listLinked.end());
			}
		} while (m_areas.SelectNext());
	}

	// restore all 3 modes (StdOut, StdErr, FileOut)

	g_vars.m_log << CLog::Pop() << CLog::Pop() << CLog::Pop();
}
