#include "pch.h"

#include "Mem.hpp"
#include "Common.hpp"
#include "CfgParse.hpp"
#include "Misc.hpp"
#include "Links.hpp"
#include "Areas.h"
#include "Msg.hpp"
#include "Tic.hpp"
#include "crc32.hpp"
#include "Temp.hpp"
#include "File.h"

#include "CfgLexems.hpp"
#include "WELexems.h"

#include "MsgAttrs.h"

#include "Adrenalin.h"

/**
 * Implementation of ProcessFixRequests
 */

void CAdrenalin::ProcessFixRequests(bool flagByHand /* = false */) {
    TCHAR   processStartTime[64];
    GetTimeForLog( processStartTime );

    WIN32_FIND_DATA FindFileData;
	HANDLE          hFindFile;
	BOOL            fFileFound;
	TCHAR           szFindFileMask[MAX_PATH];
	TCHAR           szFullFileName[MAX_PATH];
	TCHAR           szLogLine[MAX_PATH];
	int             nMailPathLen;
	int             nProcessedMsgNumber;
	vector<tstring>	vsMsgFileNames;
	CMsg		msg;
	vector<tstring>	vsBodyLines;
	TCHAR           szLine[LINE_SIZE];
	TCHAR           szReplySubject[72];
	int             i;
	int             nSubscribeNumber, nFoundNumber;
	bool            fLinksChanged;
	bool            fNewMessagesCreated; // if created we should create FixFlag
    vector<tstring> requestsLogData;

	lstrcpy( szFindFileMask, g_vars.m_szNetmailPath );
	lstrcpy( szFullFileName, g_vars.m_szNetmailPath );
	nMailPathLen = lstrlen( g_vars.m_szNetmailPath );
	lstrcpy( szFindFileMask + nMailPathLen, _T("*.msg") );

	nProcessedMsgNumber = 0;

	if (flagByHand) {
        g_vars.m_log << CLog::ResetSingle(new CLog::StdErr())
                     << TEXT("Processing Fix requests from command line:\n")
                     << CLog::Pop();
        g_vars.m_log << CLog::ResetSingle(new CLog::StdOut());
		// set sender
		if (!ParseLinkAddr( m_handCommands.m_sAddr.c_str(), msg.m_addrTo )) {
			g_vars.m_log << TEXT("[!] Error: failed to parse link address\n")
                         << CLog::Pop();
			return;
		}
		// check link
		if (!m_links.Select(msg.m_addrTo)) {
			g_vars.m_log << TEXT("[!] Error: unknown link\n")
                         << CLog::Pop();
			return;
		}
		// set sender
		strcpy( msg.m_Header.m_szToUserName, "SysOp" );
		// set receiver (we well reply)
		msg.m_addrFrom = (m_links.GetSelected().m_iOurAddr == MAIN_ADDRESS)
				? g_vars.m_addrMain 
				: g_vars.m_vAKAAddr[m_links.GetSelected().m_iOurAddr];
		strcpy( msg.m_Header.m_szFromUserName, "Adrenalin" );
		msg.Reply( ATTR_PRIVATE | ATTR_LOCAL, 0, m_links.GetSelected().m_szPassword, m_handCommands.m_vsCommands );
		vsMsgFileNames.push_back( msg.m_szLastReplyFileName );
	}
	else {
		g_vars.m_log << CLog::ResetSingle(new CLog::StdErr())
			     << TEXT("Processing Fix requests in directory ")
			     << g_vars.m_szNetmailPath << TEXT(" : \n")
			     << CLog::Pop();

		hFindFile = FindFirstFile( szFindFileMask, &FindFileData );
		fFileFound = (hFindFile != INVALID_HANDLE_VALUE);
		while (fFileFound) {
		    vsMsgFileNames.push_back( g_vars.m_szNetmailPath );
		    vsMsgFileNames.back() += FindFileData.cFileName;
		    fFileFound = FindNextFile( hFindFile, &FindFileData );
		}
		if (hFindFile != INVALID_HANDLE_VALUE)
		FindClose( hFindFile );
	}
    
	// setup log
	g_vars.m_log << CLog::SetSingle(new CLog::FileOut());

    fLinksChanged       = false;
    fNewMessagesCreated = false;
    vector<tstring>::iterator	itCurMsgFileName = vsMsgFileNames.begin();
    while (itCurMsgFileName != vsMsgFileNames.end()) {
        if (g_vars.m_fLogMsgNames) {
		    g_vars.m_log << TEXT("* msg: ") 
			             << (*itCurMsgFileName).c_str() 
        			     << TEXT_EOL;
        }
        if (msg.Read( (*itCurMsgFileName).c_str() )) {
            if (g_vars.m_fLogMsgNames) {
		        g_vars.m_log << TEXT("       read\n");
            }
            if (g_vars.m_fKillProcessedMail)
                msg.Delete();
            else
                msg.SaveAsReceived();
            // check link
            if (m_links.Select(msg.m_addrFrom)) {
		    bool	fToUs = false;
		    if (m_links.GetSelected().m_iOurAddr == MAIN_ADDRESS) {
			    fToUs = (AddrCmp( msg.m_addrTo, g_vars.m_addrMain ) == 0);
		    }
		    else {
			    fToUs = (AddrCmp( msg.m_addrTo, 
				g_vars.m_vAKAAddr[m_links.GetSelected().m_iOurAddr] ) == 0);
		    }
                if (!fToUs) {
                    // ignore message;
                    // only add line into log file
                    g_vars.m_log << _T("  message not for corresponding AKA:\r\n");
                    g_vars.m_log << _T("    from ");
		            msg.m_addrFrom.ToString( szLogLine );
		            g_vars.m_log << szLogLine;
		            msg.m_addrTo.ToString( szLogLine );
		            g_vars.m_log << _T(" to ") << szLogLine << _T("\r\n");
                }
                else if (m_links.GetSelected().m_nActivity == ACTIVITY_UNAVAILABLE) {
                    // unavailable system
                    _stprintf( szLine, m_language[LNG_UNAVAILABLE_INFO],
                             g_vars.m_szSysOp );
                    vsBodyLines.push_back( szLine );
                    msg.Reply( g_vars.m_nReplyMessageAttrs, 
			       g_vars.m_nReplyMessageFlags,
                               m_language[LNG_UNAVAILABLE_INFO_SUBJ],
                               vsBodyLines );
                    fNewMessagesCreated = true;
                    vsBodyLines.clear();
                    g_vars.m_log << _T("  message from unavailable system:\r\n    ");
		            msg.m_addrFrom.ToString( szLogLine );
                    g_vars.m_log << szLogLine << _T("\r\n");
                }
                else if (lstrcmpi( m_links.GetSelected().m_szPassword, msg.m_szSubject ) == 0)
                {
                    if (g_vars.m_vMsgData.empty()) {
                        // no commands used
                        vsBodyLines.push_back( m_language[LNG_NO_COMMANDS_USED] );
                        msg.Reply( g_vars.m_nReplyMessageAttrs,
				   g_vars.m_nReplyMessageFlags,
                                   m_language[LNG_NO_COMMANDS_USED_SUBJ],
                                   vsBodyLines );
                        fNewMessagesCreated = true;
                        vsBodyLines.clear();
                    }
                    else {
                        // process unknown commands
                        vsBodyLines.push_back( m_language[LNG_UNKNOWN_COMMANDS] );
                        nSubscribeNumber = 0;
                        vector<SMsgData>::iterator	itCurMsgData = g_vars.m_vMsgData.begin();
                        while (itCurMsgData != g_vars.m_vMsgData.end()) {
                            if ((*itCurMsgData).m_iType == MSGDATA_COMMAND &&
                                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("help")) != 0 &&
                                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("list")) != 0 &&
                                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("query")) != 0 &&
                                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("linked")) != 0 &&
                                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("unlinked")) != 0 &&
                                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("lock")) != 0 &&
                                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("unlock")) != 0 &&
				                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), TEXT("pause")) != 0 &&
				                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), TEXT("resume")) != 0 &&
				                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), TEXT("resend")) != 0 &&
				                lstrcmpi( (*itCurMsgData).m_sLine.c_str(), TEXT("avail")) != 0)
                            {
                                _stprintf( szLine, _T("  %%%s"), (*itCurMsgData).m_sLine.c_str() );
                                vsBodyLines.push_back( szLine );
                                nSubscribeNumber++;
                                (*itCurMsgData).m_iType = MSGDATA_NONE;
                            }
                            itCurMsgData++;
                        }
                        if (nSubscribeNumber > 0) {
                            msg.Reply( g_vars.m_nReplyMessageAttrs,
				                       g_vars.m_nReplyMessageFlags,
                                       m_language[LNG_UNKNOWN_COMMANDS_SUBJ],
                                       vsBodyLines );
                            fNewMessagesCreated = true;
                        }
                        vsBodyLines.clear();
                        // process known commands and subscriptions
                        vsBodyLines.push_back( m_language[LNG_SUBSCRIPTION_CHANGES] );
                        nSubscribeNumber = 0;
                        itCurMsgData = g_vars.m_vMsgData.begin();
                        // store recipient address in requests log if have any requests
                        if (itCurMsgData != g_vars.m_vMsgData.end()) {
                            lstrcpy( szLogLine, TEXT("Processing requests from ") );
                            msg.m_addrFrom.ToString( szLogLine + lstrlen(szLogLine) );
                            requestsLogData.push_back( szLogLine );
                        }
                        // go through all commands
                        while (itCurMsgData != g_vars.m_vMsgData.end()) {
                            switch ((*itCurMsgData).m_iType) {

                            case MSGDATA_SUBSCRIBE:

                                // logging request
                                _stprintf( szLogLine, TEXT("Subscribing to '%s'"),
                                           (*itCurMsgData).m_sLine.c_str() );
                                requestsLogData.push_back( szLogLine );

                                nSubscribeNumber++;
                                nFoundNumber = 0;
                                if (m_areas.SelectFirst()) {
					                do {
					                    if (CheckMask( m_areas.GetSelectedArea()->m_szName,
							                 const_cast<TCHAR*>((*itCurMsgData).m_sLine.c_str()) ))
					                    {
						                    if (m_areas.Linked( &m_links.GetSelected() )) {
                                                // already linked
						                        _stprintf( szLine, m_language[LNG_AREA_ALREADY_LINKED],
							                               m_areas.GetSelectedArea()->m_szName );
                                                _stprintf( szLogLine, TEXT( "Already linked to '%s'" ),
                                                           m_areas.GetSelectedArea()->m_szName );
                                                requestsLogData.push_back( szLogLine );
						                    }
						                    else {
                                                if (m_areas.Link( &m_links.GetSelected() )) {
                                                    // linked
							                        _stprintf( szLine, m_language[LNG_AREA_LINKED],
								                    m_areas.GetSelectedArea()->m_szName );
                                                    _stprintf( szLogLine, TEXT( "Linked to '%s'" ),
                                                               m_areas.GetSelectedArea()->m_szName );
                                                    requestsLogData.push_back( szLogLine );
                                                }
                                                else {
                                                    // no access
							                        _stprintf( szLine, m_language[LNG_AREA_NO_ACCESS],
								                    m_areas.GetSelectedArea()->m_szName );
                                                    _stprintf( szLogLine, TEXT( "No access to '%s'" ),
                                                               m_areas.GetSelectedArea()->m_szName );
                                                    requestsLogData.push_back( szLogLine );
                                                }
						                    }
						                    vsBodyLines.push_back( szLine );
						                    nFoundNumber++;
					                    }					    
					                } while (m_areas.SelectNext());
				                }
                                if (nFoundNumber == 0) {
					                if (IsMask(const_cast<TCHAR*>((*itCurMsgData).m_sLine.c_str()))) {
                                        // no area sutisfies mask
						                _stprintf( szLine, m_language[LNG_AREA_MASK_NOT_FOUND],
							                (*itCurMsgData).m_sLine.c_str() );
						                vsBodyLines.push_back( szLine );
                                        _stprintf( szLogLine, TEXT("No area found for mask '%s'"),
                                                   (*itCurMsgData).m_sLine.c_str() );
                                        requestsLogData.push_back( szLogLine );
					                }
					            else {
						char	szOemLine[LINE_SIZE];
						// trying to forward request
						if (m_links.SelectFirst()) {
							do {
								// skip requestor
								if (AddrCmp( m_links.GetSelected().m_addr, msg.m_addrFrom ) == 0) {
									if (m_links.SelectNext())
										continue;
									break;
								}
								// check AvailAreas
								if (m_links.GetSelected().m_szAvailAreasFileName[0] == _T('\0')) {
									if (m_links.SelectNext())
										continue;
									break;
								}
								// open areas file
								char	szOemFileName[MAX_PATH];
								CharToOem( m_links.GetSelected().m_szAvailAreasFileName, szOemFileName );
								ifstream	in(szOemFileName);
								if (in.fail()) {
									if (m_links.SelectNext())
										continue;
									break;
								}
								bool	fAreaFound = false;
								while (!in.eof() && !fAreaFound) {
									in.getline( szOemLine, LINE_SIZE );
									if (in.fail())
										break;
									OemToChar( szOemLine, szLine );
									fAreaFound = (lstrcmpi( szLine, (*itCurMsgData).m_sLine.c_str() ) == 0);
								}
								in.close();
								if (fAreaFound)
									break;
							} while (m_links.SelectNext());
						}
						if (m_links.IsSelected()) {
							SLink&		link = m_links.GetSelected();
							// link to forward found
                            _stprintf( szLogLine, TEXT("Forwarding request for '%s' to "),
                                       (*itCurMsgData).m_sLine.c_str() );
                            link.m_addr.ToString( szLogLine + lstrlen(szLogLine) );
                            requestsLogData.push_back( szLogLine );
							// write message to this link
							CMsg		forwardMsg;
							vector<tstring>	vsForwardBody;
							// print log message
							g_vars.m_log << _T("Forwarding request from ");
							msg.m_addrFrom.ToString( szLine );
							g_vars.m_log << szLine << _T(" to ");
							link.m_addr.ToString( szLine );
							g_vars.m_log << szLine << _T("\r\n\011requesting ") << (*itCurMsgData).m_sLine.c_str() << _T("\r\n");

							_stprintf( szLine, _T("+%s"), (*itCurMsgData).m_sLine.c_str() );
							vsForwardBody.push_back( szLine );
							MakeDirStructure( g_vars.m_szNetmailPath );
							// from/to fields reversed because we will use reply method
							forwardMsg.m_addrFrom = link.m_addr;
							if (link.m_iOurAddr == MAIN_ADDRESS)
								forwardMsg.m_addrTo = g_vars.m_addrMain;
							else
								forwardMsg.m_addrTo = g_vars.m_vAKAAddr[link.m_iOurAddr];
							if (link.m_szRobotName[0] != _T('\0'))
								CharToOem( link.m_szRobotName, forwardMsg.m_Header.m_szFromUserName );
							else
								strcpy( forwardMsg.m_Header.m_szFromUserName, "AllFix" );
							CharToOem( g_vars.m_szSysOp, forwardMsg.m_Header.m_szToUserName );
							forwardMsg.m_lFreeMsgNumber = msg.m_lFreeMsgNumber;
							forwardMsg.Reply( ATTR_PRIVATE | ATTR_LOCAL, 0, link.m_szPassword, vsForwardBody );
							msg.m_lFreeMsgNumber++;
							g_vars.m_log << _T("Forward request message created.\r\n");
							// add line
							_stprintf( szLine, m_language[LNG_REQUEST_FORWARDED],
								(*itCurMsgData).m_sLine.c_str() );
							vsBodyLines.push_back( szLine );
							// 2. Let's keep track of requestors
							if (!m_fForwardsRead) {
								LoadForwards();
							}
							// 3. Add new forward info, format of date is DDMMYY
							SForwardInfo	fwdInfo;
							SYSTEMTIME	systemTime;
							GetLocalTime( &systemTime );
							_stprintf( fwdInfo.m_szDate, _T("%02d%02d%02d"), systemTime.wDay, systemTime.wMonth, systemTime.wYear % 100 );
							_stprintf( fwdInfo.m_szLink, _T("%d:%d/%d.%d@%s"), msg.m_addrFrom.Zone, msg.m_addrFrom.Net, msg.m_addrFrom.Node, msg.m_addrFrom.Point, msg.m_addrFrom.Domain );
							lstrcpy( fwdInfo.m_szArea, (*itCurMsgData).m_sLine.c_str() );
							m_vectForwardInfo.push_back( fwdInfo );
							m_fForwardsChanged = true;
						}
						else {
                            // area not found
							_stprintf( szLine, m_language[LNG_AREA_NOT_FOUND],
								(*itCurMsgData).m_sLine.c_str() );
							vsBodyLines.push_back( szLine );
                            _stprintf( szLogLine, TEXT("Not found '%s'"), (*itCurMsgData).m_sLine.c_str() );
                            requestsLogData.push_back( szLogLine );
						}
						// restore selection
						m_links.Select( msg.m_addrFrom );
					}
                                    
                                }
                                break;

                            case MSGDATA_UNSUBSCRIBE:

                                // logging request
                                _stprintf( szLogLine, TEXT("Unsubscribing from '%s'"),
                                           (*itCurMsgData).m_sLine.c_str() );
                                requestsLogData.push_back( szLogLine );

                                nSubscribeNumber++;
                                nFoundNumber = 0;
                                if (m_areas.SelectFirst()) {
					do {
					    if (CheckMask( m_areas.GetSelectedArea()->m_szName,
							   const_cast<TCHAR*>((*itCurMsgData).m_sLine.c_str())))
					    {
						if (m_areas.Unlink( &m_links.GetSelected() )) {
                            // unlinked
						    _stprintf( szLine, m_language[LNG_AREA_UNLINKED],
							     m_areas.GetSelectedArea()->m_szName );
						    vsBodyLines.push_back( szLine );
						    nFoundNumber++;
                            _stprintf( szLogLine, TEXT("Unlinked from '%s'"), m_areas.GetSelectedArea()->m_szName );
                            requestsLogData.push_back( szLogLine );
						}
						else if (!IsMask(const_cast<TCHAR*>((*itCurMsgData).m_sLine.c_str()))) {
                            // not linked
						    _stprintf( szLine, m_language[LNG_AREA_NOT_LINKED],
							     m_areas.GetSelectedArea()->m_szName );
						    vsBodyLines.push_back( szLine );
						    nFoundNumber++;
                            _stprintf( szLogLine, TEXT("Was not linked to '%s'"), m_areas.GetSelectedArea()->m_szName );
                            requestsLogData.push_back( szLogLine );
						}
					    }
					    
					} while (m_areas.SelectNext());
				}
                                if (nFoundNumber == 0) {
                                    if (IsMask((*itCurMsgData).m_sLine.c_str())) {
                                        _stprintf( szLine, m_language[LNG_AREA_MASK_NOT_FOUND],
                                                 (*itCurMsgData).m_sLine.c_str() );
                                        _stprintf( szLogLine, TEXT("No area found for mask '%s'"),
                                                   (*itCurMsgData).m_sLine.c_str() );
                                    }
                                    else {
                                        _stprintf( szLine, m_language[LNG_AREA_NOT_FOUND],
                                                 (*itCurMsgData).m_sLine.c_str() );
                                        _stprintf( szLogLine, TEXT("Not found '%s'"),
                                                   (*itCurMsgData).m_sLine.c_str() );
                                    }
                                    vsBodyLines.push_back( szLine );
                                    requestsLogData.push_back( szLogLine );
                                }
                                break;

                            case MSGDATA_COMMAND:

                                // logging request
                                _stprintf( szLogLine, TEXT("Requesting command '%s', tail = '%s'"),
                                           (*itCurMsgData).m_sLine.c_str(), (*itCurMsgData).m_sTail.c_str() );
                                requestsLogData.push_back( szLogLine );

                                if (nSubscribeNumber > 0) {
                                    msg.Reply( g_vars.m_nReplyMessageAttrs,
					                           g_vars.m_nReplyMessageFlags,
                                               m_language[LNG_SUBSCRIPTION_CHANGES_SUBJ],
                                               vsBodyLines );
                                    fNewMessagesCreated = true;
                                    nSubscribeNumber = 0;
                                }
                                vsBodyLines.clear();
                                vsBodyLines.push_back(_T(""));
                                if (lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("help") ) == 0) {
                                    lstrcpy( szReplySubject, m_language[LNG_HELP_SUBJ] );
                                    vsBodyLines.push_back( m_language[LNG_HELP_HEAD] );
                                    if (!g_vars.m_vsAliases.empty()) {
                                        vsBodyLines.push_back( m_language[LNG_HELP_ALIASES] );
                                        for (i = 1; i < g_vars.m_vsAliases.size(); i++) {
                                            _stprintf( szLine, _T("  %s"), g_vars.m_vsAliases[i] );
                                            vsBodyLines.push_back( szLine );
                                        }
					                    vsBodyLines.push_back( _T("") );
                                    }
                                    vsBodyLines.push_back( m_language[LNG_HELP] );
                                    requestsLogData.push_back( TEXT("Help sent") );
                                }
                                else if (lstrcmpi( (*itCurMsgData).m_sLine.c_str(), TEXT("resend") ) == 0) {

									// RESEND command

									lstrcpy( szReplySubject, m_language[LNG_RESEND_SUBJECT] );
									ProcessResendCommand( (*itCurMsgData), vsBodyLines );

								}
                                else if (lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("list") ) == 0) {
					                // answer on %list
                                    lstrcpy( szReplySubject, m_language[LNG_LIST_SUBJ] );
                                    i = 0;
                                    if (m_areas.SelectFirst()) {
					                    do {
						                    if (m_areas.CanAccess( &m_links.GetSelected() ))
						                        i++;
					                    } while (m_areas.SelectNext());
				                    }
                                    if (i == 0)
                                        vsBodyLines.push_back( m_language[LNG_LIST_EMPTY] );
                                    else {
					                    _stprintf( szLine, m_language[LNG_LIST_TITLE], i );
					                    vsBodyLines.push_back( szLine );
					                    // count groups
					                    ULONG	ulGroup = 0;
					                    if (m_areas.SelectFirst()) {
						                    do {
							                    if (m_areas.CanAccess( &m_links.GetSelected() )) {
							    SArea*	pArea = m_areas.GetSelectedArea();
							    if (pArea->m_ulGroup != 0 && pArea->m_ulGroup != ulGroup) {
								    // remember current group
								    ulGroup = pArea->m_ulGroup;
								    // print group name if present
								    tstring&	sGroupName = m_areas.GetGroupName(ulGroup);
								    if (sGroupName.length() > 0) {
									    TCHAR	cGroup = _T('A');
									    ULONG	ulValue = ulGroup;
									    while ((ulValue & 1) == 0) {
										    ulValue >>= 1;
										    cGroup++;
									    }
									    _stprintf( szLine, _T("  %s %c: %s"), m_language[LNG_GROUP], cGroup, sGroupName.c_str() );
									    vsBodyLines.push_back( _T("") );
									    vsBodyLines.push_back( _T("  ============================================================================") );
									    vsBodyLines.push_back( szLine );
									    vsBodyLines.push_back( _T("  ============================================================================") );
									    vsBodyLines.push_back( _T("") );
								    }
								    else {
									    vsBodyLines.push_back( _T("") );
								    }
							    }
							    m_areas.GetNameDesc( szLine );
							    if (m_areas.Linked( &m_links.GetSelected() )) {
								switch (m_areas.GetRouteType( &m_links.GetSelected())) {
								case ROUTE_RECEIVE:
								    szLine[0] = _T('R');
								    break;
								case ROUTE_SEND:
								    szLine[0] = _T('S');
								    break;
								case ROUTE_BIDIRECT:
								    szLine[0] = _T('B');
								    break;
								}
							    }
							    vsBodyLines.push_back( szLine );
							}						
						    } while (m_areas.SelectNext());
					    }
					                    vsBodyLines.push_back( m_language[LNG_LIST_MARKS_TITLE] );
					                    vsBodyLines.push_back( m_language[LNG_MARKS] );
                                    }
                                    requestsLogData.push_back( TEXT("List of accessible areas sent") );
                                }
                                else if (lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("query") ) == 0 ||
                                         lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("linked") ) == 0)
                                {
					// answer on %query or %linked
                                    lstrcpy( szReplySubject, m_language[LNG_LINKED_SUBJ] );
                                    i = 0;
                                    if (m_areas.SelectFirst()) {
					    do {
						if (m_areas.Linked( &m_links.GetSelected() ))
						    i++;						
					    } while (m_areas.SelectNext());
				    }
                                    if (i == 0)
                                        vsBodyLines.push_back( m_language[LNG_LINKED_EMPTY] );
                                    else {
                                        _stprintf( szLine, m_language[LNG_LINKED_TITLE], i );
                                        vsBodyLines.push_back( szLine );
					    // count groups
					    ULONG	ulGroup = 0;
					    if (m_areas.SelectFirst()) {
						    do {
							if (m_areas.Linked( &m_links.GetSelected() )) {
							    SArea*	pArea = m_areas.GetSelectedArea();
							    if (pArea->m_ulGroup != 0 && pArea->m_ulGroup != ulGroup) {
								    // remember current group
								    ulGroup = pArea->m_ulGroup;
								    // print group name if present
								    tstring&	sGroupName = m_areas.GetGroupName(ulGroup);
								    if (sGroupName.length() > 0) {
									    TCHAR	cGroup = _T('A');
									    ULONG	ulValue = ulGroup;
									    while ((ulValue & 1) == 0) {
										    ulValue >>= 1;
										    cGroup++;
									    }
									    _stprintf( szLine, _T("  %s %c: %s"), m_language[LNG_GROUP], cGroup, sGroupName.c_str() );
									    vsBodyLines.push_back( _T("") );
									    vsBodyLines.push_back( _T("  ============================================================================") );
									    vsBodyLines.push_back( szLine );
									    vsBodyLines.push_back( _T("  ============================================================================") );
									    vsBodyLines.push_back( _T("") );
								    }
								    else {
									    vsBodyLines.push_back( _T("") );
								    }
							    }
							    m_areas.GetNameDesc( szLine );
							    if (m_areas.Linked( &m_links.GetSelected() )) {
								switch (m_areas.GetRouteType( &m_links.GetSelected() )) {
								case ROUTE_RECEIVE:
								    szLine[0] = _T('R');
								    break;
								case ROUTE_SEND:
								    szLine[0] = _T('S');
								    break;
								case ROUTE_BIDIRECT:
								    szLine[0] = _T('B');
								    break;
								}
							    }
							    vsBodyLines.push_back( szLine );
							}
						    } while (m_areas.SelectNext());
					    }
                                        vsBodyLines.push_back( m_language[LNG_LINKED_MARKS_TITLE] );
                                        vsBodyLines.push_back( m_language[LNG_MARKS] );
                                    }
                                    requestsLogData.push_back( TEXT("List of linked areas sent") );
                                }
                                else if (lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("unlinked") ) == 0) {
					// answer on %unlinked
                                    lstrcpy( szReplySubject, m_language[LNG_LINKED_SUBJ] );
                                    i = 0;
				    // count groups
				    ULONG	ulGroup = 0;
                                    if (m_areas.SelectFirst()) {
					    do {
						if (m_areas.CanAccess( &m_links.GetSelected()) &&
						    !m_areas.Linked( &m_links.GetSelected() ))
						    i++;
					    } while (m_areas.SelectNext());
				    }
                                    if (i == 0)
                                        vsBodyLines.push_back( m_language[LNG_UNLINKED_EMPTY]);
                                    else {
                                        _stprintf( szLine, m_language[LNG_UNLINKED_TITLE], i );
                                        vsBodyLines.push_back( szLine );
                                    }
                                    if (m_areas.SelectFirst()) {
					    do {
						if (m_areas.CanAccess( &m_links.GetSelected()) &&
						    !m_areas.Linked( &m_links.GetSelected() ))
						{
						    SArea*	pArea = m_areas.GetSelectedArea();
						    if (pArea->m_ulGroup != 0 && pArea->m_ulGroup != ulGroup) {
							    // remember current group
							    ulGroup = pArea->m_ulGroup;
							    // print group name if present
							    tstring&	sGroupName = m_areas.GetGroupName(ulGroup);
							    if (sGroupName.length() > 0) {
								    TCHAR	cGroup = _T('A');
								    ULONG	ulValue = ulGroup;
								    while ((ulValue & 1) == 0) {
									    ulValue >>= 1;
									    cGroup++;
								    }
								    _stprintf( szLine, _T("  %s %c: %s"), m_language[LNG_GROUP], cGroup, sGroupName.c_str() );
								    vsBodyLines.push_back( _T("") );
								    vsBodyLines.push_back( _T("  ============================================================================") );
								    vsBodyLines.push_back( szLine );
								    vsBodyLines.push_back( _T("  ============================================================================") );
								    vsBodyLines.push_back( _T("") );
							    }
							    else {
								    vsBodyLines.push_back( _T("") );
							    }
						    }
						    m_areas.GetNameDesc( szLine );
						    vsBodyLines.push_back( szLine );
						}						
					    } while (m_areas.SelectNext());
				    }
                                    requestsLogData.push_back( TEXT("List of not linked areas sent") );
                                }
                                else if (lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("lock") ) == 0 ||
					 lstrcmpi( (*itCurMsgData).m_sLine.c_str(), TEXT("pause") ) == 0) {
                                    lstrcpy( szReplySubject, m_language[LNG_LOCK_SUBJ] );
                                    if (m_links.GetSelected().m_nActivity == ACTIVITY_USUAL) {
                                        m_links.GetSelected().m_nActivity = ACTIVITY_PASSIVE;
                                        fLinksChanged = true;
                                        vsBodyLines.push_back( m_language[LNG_LOCK] );
                                        requestsLogData.push_back( TEXT("Locked") );
                                    }
                                    else {
                                        vsBodyLines.push_back( m_language[LNG_ALREADY_LOCKED] );
                                        requestsLogData.push_back( TEXT("Already locked") );
                                    }
                                    vsBodyLines.push_back( m_language[LNG_UNLOCK_NOTIFY] );
                                }
                                else if (lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("unlock") ) == 0 ||
					 lstrcmpi( (*itCurMsgData).m_sLine.c_str(), _T("resume") ) == 0) {
                                    lstrcpy( szReplySubject, m_language[LNG_UNLOCK_SUBJ] );
                                    if (m_links.GetSelected().m_nActivity == ACTIVITY_PASSIVE) {
                                        m_links.GetSelected().m_nActivity = ACTIVITY_USUAL;
                                        fLinksChanged = true;
                                        vsBodyLines.push_back( m_language[LNG_UNLOCK] );
                                        requestsLogData.push_back( TEXT("Unlocked") );
                                    }
                                    else {
                                        vsBodyLines.push_back( m_language[LNG_NOT_LOCKED] );
                                        requestsLogData.push_back( TEXT("Was not locked") );
                                    }
                                    vsBodyLines.push_back( m_language[LNG_LOCK_NOTIFY] );
                                }
				else if (lstrcmpi( (*itCurMsgData).m_sLine.c_str(), TEXT("avail") ) == 0) {
					/*******
					  AVAIL
					 *******/
					lstrcpy( szReplySubject, m_language[LNG_AVAIL_LIST_SUBJECT] );					
					if (m_links.SelectFirst()) {
						char		oemLine[LINE_SIZE];
						VTString	availAreas;
						do {
							// skip requestor
							if (AddrCmp( m_links.GetSelected().m_addr, msg.m_addrFrom ) == 0) {
								continue;
							}
							// check AvailAreas
							if (m_links.GetSelected().m_szAvailAreasFileName[0] == TEXT('\0')) {
								continue;
							}
							// open areas file
							char	szOemFileName[MAX_PATH];
							CharToOem( m_links.GetSelected().m_szAvailAreasFileName, szOemFileName );
							ifstream	in(szOemFileName);
							if (in.fail()) {
								continue;
							}
							while (!in.eof()) {
								in.getline( oemLine, LINE_SIZE );
								if (in.fail())
									break;
								OemToChar( oemLine, szLine );
								// keep only nonempty lines
								if (*szLine != TEXT('\0')) {
									availAreas.push_back( szLine );
								}
							}
							in.close();
						} while (m_links.SelectNext());
						// check number of available areas
						if (availAreas.empty()) {
							// no available areas
							vsBodyLines.push_back( m_language[LNG_AVAIL_LIST_EMPTY] );
						}
						else {
							vsBodyLines.push_back( m_language[LNG_AVAIL_LIST_HEAD] );
							AppendVector( vsBodyLines, availAreas );
						}
					}
                    requestsLogData.push_back( TEXT("List of uplink-available areas sent") );
				}
                                msg.Reply( g_vars.m_nReplyMessageAttrs,
					   g_vars.m_nReplyMessageFlags,
                                           szReplySubject, vsBodyLines );
                                fNewMessagesCreated = true;
                                vsBodyLines.clear();
                                vsBodyLines.push_back( m_language[LNG_SUBSCRIPTION_CHANGES] );
                                nSubscribeNumber = 0;
                                break;
                            };
                            itCurMsgData++;
                        }
                        if (nSubscribeNumber > 0) {
                            msg.Reply( g_vars.m_nReplyMessageAttrs,
				       g_vars.m_nReplyMessageFlags,
                                       m_language[LNG_SUBSCRIPTION_CHANGES_SUBJ],
                                       vsBodyLines );
                            fNewMessagesCreated = true;
                            nSubscribeNumber = 0;
                        }
                    }
                }
                else {
                    // incorrect password
                    _stprintf( szLine, m_language[LNG_INCORRECT_PASSWORD],
                             msg.m_szSubject, g_vars.m_szSysOp );
                    vsBodyLines.push_back( szLine );
                    msg.Reply( g_vars.m_nReplyMessageAttrs,
			       g_vars.m_nReplyMessageFlags,
                               m_language[LNG_ACCESS_DENIED_SUBJ],
                               vsBodyLines );
                    fNewMessagesCreated = true;
                    vsBodyLines.clear();
                }
            }
            else {
                // link is not listed
                _stprintf( szLine, m_language[LNG_UNKNOWN_USER],
                         msg.m_szToUserName, g_vars.m_szSysOp );
                vsBodyLines.push_back( szLine );
                msg.Reply( g_vars.m_nReplyMessageAttrs,
			   g_vars.m_nReplyMessageFlags,
                           m_language[LNG_ACCESS_DENIED_SUBJ],
                           vsBodyLines );
                fNewMessagesCreated = true;
                vsBodyLines.clear();
            }
            g_vars.FreeMsgData();
            nProcessedMsgNumber++;
        }
        else {
            if (g_vars.m_fLogMsgNames)
                g_vars.m_log << TEXT("       failed\n");
        }
	itCurMsgFileName++;
    }
    if (fNewMessagesCreated && *g_vars.m_szFixFlagName != _T('\0'))
        CreateFileFlag( g_vars.m_szFixFlagName );
    if (fLinksChanged)
        SaveCfg();

    g_vars.m_log << CLog::ResetSingle(new CLog::StdErr())
                 << TEXT("Fix done.\n")
                 << CLog::Pop();

    // restore log modes
    g_vars.m_log << CLog::Pop();

    // write requests log
    if (!IsEmptyStr( requestsLogFileName ) && !requestsLogData.empty()) {
        CFile   requestsLog;
        if (requestsLog.Open( requestsLogFileName, GENERIC_WRITE, OPEN_ALWAYS )) {
            requestsLog.SeekToEnd();
            requestsLog.WriteStringAsOem( processStartTime );
            requestsLog.WriteStringAsOem( TEXT_EOL );
            for (vector<tstring>::iterator   it = requestsLogData.begin();
                 it != requestsLogData.end(); it++)
            {                
                requestsLog.WriteStringAsOem( (*it).c_str() );
                requestsLog.WriteStringAsOem( TEXT_EOL );
            }
            // get end time into the processStartTime (use the same variable)
            GetTimeForLog( processStartTime );
            requestsLog.WriteStringAsOem( processStartTime );
            requestsLog.WriteStringAsOem( TEXT_EOL );
            requestsLog.WriteStringAsOem( TEXT_EOL );
            requestsLog.Close();
        }
        else {
            g_vars.m_log << CLog::ResetSingle( new CLog::StdOut() )
                         << TEXT("Failed to open requests log file: ")
                         << requestsLogFileName
                         << TEXT_EOL
                         << CLog::Pop();
        }
    }
}
